@@ -177,8 +177,10 @@ safe:
 	if (needed_workers > 0) {
 		for (i = 1; i <= uwsgi.numproc; i++) {
 			if (uwsgi.workers[i].cheaped == 1 && uwsgi.workers[i].pid == 0) {
-				if (uwsgi_respawn_worker(i))
+				if (uwsgi_respawn_worker(i)) {
+					uwsgi.cheaper_fifo_delta += needed_workers;
 					return 0;
+				}
 				needed_workers--;
 			}
 			if (needed_workers == 0)
@@ -186,27 +188,34 @@ safe:
 		}
 	}
 	else if (needed_workers < 0) {
-		int oldest_worker = 0;
-		time_t oldest_worker_spawn = INT_MAX;
-		for (i = 1; i <= uwsgi.numproc; i++) {
-			if (uwsgi.workers[i].cheaped == 0 && uwsgi.workers[i].pid > 0) {
-				if (uwsgi_worker_is_busy(i) == 0) {
-					if (uwsgi.workers[i].last_spawn < oldest_worker_spawn) {
-						oldest_worker_spawn = uwsgi.workers[i].last_spawn;
-						oldest_worker = i;
+		while (needed_workers < 0) {
+			int oldest_worker = 0;
+			time_t oldest_worker_spawn = INT_MAX;
+			for (i = 1; i <= uwsgi.numproc; i++) {
+				if (uwsgi.workers[i].cheaped == 0 && uwsgi.workers[i].pid > 0) {
+					if (uwsgi_worker_is_busy(i) == 0) {
+						if (uwsgi.workers[i].last_spawn < oldest_worker_spawn) {
+							oldest_worker_spawn = uwsgi.workers[i].last_spawn;
+							oldest_worker = i;
+						}
 					}
 				}
 			}
-		}
-		if (oldest_worker > 0) {
+			if (oldest_worker > 0) {
 #ifdef UWSGI_DEBUG
-			uwsgi_log("worker %d should die...\n", oldest_worker);
+				uwsgi_log("worker %d should die...\n", oldest_worker);
 #endif
-			uwsgi.workers[oldest_worker].cheaped = 1;
-			uwsgi.workers[oldest_worker].rss_size = 0;
-			uwsgi.workers[oldest_worker].vsz_size = 0;
-			uwsgi.workers[oldest_worker].manage_next_request = 0;
-			uwsgi_curse(oldest_worker, SIGWINCH);
+				uwsgi.workers[oldest_worker].cheaped = 1;
+				uwsgi.workers[oldest_worker].rss_size = 0;
+				uwsgi.workers[oldest_worker].vsz_size = 0;
+				uwsgi.workers[oldest_worker].manage_next_request = 0;
+				uwsgi_curse(oldest_worker, SIGWINCH);
+			}
+			else {
+				// Return it to the pool
+				uwsgi.cheaper_fifo_delta--;
+			}
+			needed_workers++;
 		}
 	}
 
@@ -392,7 +401,20 @@ void uwsgi_reload(char **argv) {
 	int i;
 	int waitpid_status;
 
-	if (uwsgi.new_argv) argv = uwsgi.new_argv;
+	// hack for removing garbage generated by embedded loaders (like pyuwsgi)
+	if (uwsgi.new_argc) {
+		char **tmp_argv = argv;
+		for(;;) {
+			char *arg = *tmp_argv;
+			if (!arg)
+				break;
+			if (strlen(arg) == 0)
+			{
+				*tmp_argv = NULL;
+			}
+			tmp_argv++;
+		}
+	}
 
 	if (!uwsgi.master_is_reforked) {
 
