@@ -32,6 +32,7 @@ struct uwsgi_php {
 	struct uwsgi_string_list *exec_after;
 
 	char *sapi_name;
+	HashTable user_config_cache;
 } uphp;
 
 void uwsgi_opt_php_ini(char *opt, char *value, void *foobar) {
@@ -547,6 +548,76 @@ static zend_module_entry uwsgi_module_entry = {
         STANDARD_MODULE_PROPERTIES
 };
 
+typedef struct _user_config_cache_entry {
+	time_t expires;
+	HashTable *user_config;
+} user_config_cache_entry;
+
+#if (PHP_MAJOR_VERSION >= 7)
+static void user_config_cache_entry_dtor(zval *el) {
+	user_config_cache_entry *entry = (user_config_cache_entry *)Z_PTR_P(el);
+#else
+static void user_config_cache_entry_dtor(user_config_cache_entry *entry) {
+#endif
+	zend_hash_destroy(entry->user_config);
+	free(entry->user_config);
+	free(entry);
+}
+
+static void activate_user_config(const char *filename, const char *doc_root, size_t doc_root_len) {
+	char *ptr;
+	user_config_cache_entry *new_entry, *entry;
+
+	time_t request_time = (time_t)sapi_get_request_time();
+
+	// get dirname (path) from filename
+	size_t path_len = (strrchr(filename, DEFAULT_SLASH) - filename) + 1;
+	char path[path_len];
+	memcpy(path, filename, path_len);
+	path[path_len] = '\0';
+
+	// get or create entry from cache
+#if (PHP_MAJOR_VERSION >= 7)
+	if ((entry = zend_hash_str_find_ptr(&uphp.user_config_cache, path, path_len)) == NULL) {
+#else
+	if (zend_hash_find(&uphp.user_config_cache, path, path_len + 1, (void **) &entry) == FAILURE) {
+#endif
+		new_entry = pemalloc(sizeof(user_config_cache_entry), 1);
+		new_entry->expires = 0;
+		new_entry->user_config = (HashTable *) pemalloc(sizeof(HashTable), 1);
+
+		// make zend_hash to store all user.ini settings.
+		zend_hash_init(new_entry->user_config, 0, NULL, (dtor_func_t) config_zval_dtor, 1);
+#if (PHP_MAJOR_VERSION >= 7)
+		entry = zend_hash_str_update_ptr(&uphp.user_config_cache, path, path_len, new_entry);
+#else
+		zend_hash_update(&uphp.user_config_cache, path, path_len + 1, new_entry, sizeof(user_config_cache_entry), (void **) &entry);
+#endif
+	}
+
+	if (request_time > entry->expires) {
+
+		// clear the expired config
+		zend_hash_clean(entry->user_config);
+
+		// set pointer to end of docroot
+		ptr = path + (doc_root_len - 1);
+
+		// parse all user.ini files starting from docroot.
+		while ((ptr = strchr(ptr, DEFAULT_SLASH)) != NULL) {
+			*ptr = 0;
+			php_parse_user_ini_file(path, PG(user_ini_filename), entry->user_config);
+			*ptr = '/';
+			ptr++;
+		}
+
+		// set (new) expiry time
+		entry->expires = request_time + PG(user_ini_cache_ttl);
+	}
+
+	// activate all user.ini variables
+	php_ini_activate_config(entry->user_config, PHP_INI_PERDIR, PHP_INI_STAGE_HTACCESS);
+}
 
 static int php_uwsgi_startup(sapi_module_struct *sapi_module)
 {
@@ -624,6 +695,8 @@ int uwsgi_php_init(void) {
 		uwsgi_log("--- end of PHP custom config ---\n");
 	}
 
+	zend_hash_init(&uphp.user_config_cache, 0, NULL, (dtor_func_t) user_config_cache_entry_dtor, 1);
+
 	// fix docroot
         if (uphp.docroot) {
 		char *orig_docroot = uphp.docroot;
@@ -793,6 +866,7 @@ oldstyle:
 #endif
 
 	filename = uwsgi_concat4n(wsgi_req->document_root, wsgi_req->document_root_len, "/", 1, wsgi_req->path_info, wsgi_req->path_info_len, "", 0);
+	activate_user_config(filename, wsgi_req->document_root, wsgi_req->document_root_len);
 
 	if (uwsgi_php_walk(wsgi_req, filename, wsgi_req->document_root, wsgi_req->document_root_len, &path_info)) {
 		free(filename);
