@@ -148,13 +148,11 @@ char *uwsgi_encode_pydict(PyObject * pydict, uint16_t * size) {
 
 		if (!PyTuple_Check(zero)) {
 			uwsgi_log("invalid python dictionary item\n");
-			Py_DECREF(zero);
 			continue;
 		}
 
 		if (PyTuple_Size(zero) < 2) {
 			uwsgi_log("invalid python dictionary item\n");
-			Py_DECREF(zero);
 			continue;
 		}
 		key = PyTuple_GetItem(zero, 0);
@@ -167,7 +165,6 @@ char *uwsgi_encode_pydict(PyObject * pydict, uint16_t * size) {
 		}
 
 		if (!PyString_Check(key) || !PyString_Check(val)) {
-			Py_DECREF(zero);
 			continue;
 		}
 
@@ -187,8 +184,6 @@ char *uwsgi_encode_pydict(PyObject * pydict, uint16_t * size) {
 			bufptr += valsize;
 		}
 
-		Py_DECREF(zero);
-
 	}
 
 	return buf;
@@ -1986,7 +1981,6 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 					if (PyString_Check(val)) {
 						valsize = PyString_Size(val);
 						if (uwsgi_buffer_append_keyval(ub, PyString_AsString(key), keysize, PyString_AsString(val), valsize)) {
-							Py_DECREF(zero);
 							uwsgi_buffer_destroy(ub);
 							goto error;
 						}
@@ -1998,31 +1992,26 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 						PyObject *str = PyObject_Str(val);
 #endif
 						if (!str) {
-							Py_DECREF(zero);
 							uwsgi_buffer_destroy(ub);
 							goto error;
 						}
 						if (uwsgi_buffer_append_keyval(ub, PyString_AsString(key), keysize, PyString_AsString(str), PyString_Size(str))) {
-                                                        Py_DECREF(zero);
 							Py_DECREF(str);
-                                                        uwsgi_buffer_destroy(ub);
-                                                        goto error;
-                                                }
+							uwsgi_buffer_destroy(ub);
+							goto error;
+						}
 						Py_DECREF(str);
 					}
 				}
 				else {
-					Py_DECREF(zero);
 					uwsgi_buffer_destroy(ub);
                                         goto error;
 				}
 			}
 			else {
-				Py_DECREF(zero);
 				uwsgi_buffer_destroy(ub);
                                 goto error;
 			}
-			Py_DECREF(zero);
 		}
 		else {
 			uwsgi_buffer_destroy(ub);
@@ -2041,7 +2030,9 @@ PyObject *py_uwsgi_send_spool(PyObject * self, PyObject * args, PyObject *kw) {
 
 
 	if (pybody) {
-		Py_DECREF(pybody);
+		if (PyString_Check(pybody)) {
+			Py_DECREF(pybody);
+		}
 	}
 	
 	Py_DECREF(spool_vars);
@@ -2177,9 +2168,6 @@ PyObject *py_uwsgi_workers(PyObject * self, PyObject * args) {
 		}
 
 		apps_tuple = PyDict_GetItemString(worker_dict, "apps");
-		if (apps_tuple) {
-			Py_DECREF(apps_tuple);
-		}	
 
 		PyDict_Clear(worker_dict);
 
@@ -2560,11 +2548,11 @@ PyObject *py_uwsgi_parse_file(PyObject * self, PyObject * args) {
 
 static PyMethodDef uwsgi_spooler_methods[] = {
 #ifdef PYTHREE
-	{"send_to_spooler", (PyCFunction) py_uwsgi_send_spool, METH_VARARGS | METH_KEYWORDS, ""},
-	{"spool", (PyCFunction) py_uwsgi_send_spool, METH_VARARGS | METH_KEYWORDS, ""},
+	{"send_to_spooler", (PyCFunction)(void *)py_uwsgi_send_spool, METH_VARARGS|METH_KEYWORDS, ""},
+	{"spool", (PyCFunction)(void *)py_uwsgi_send_spool, METH_VARARGS|METH_KEYWORDS, ""},
 #else
-	{"send_to_spooler", (PyCFunction) py_uwsgi_send_spool, METH_KEYWORDS, ""},
-	{"spool", (PyCFunction) py_uwsgi_send_spool, METH_KEYWORDS, ""},
+	{"send_to_spooler", (PyCFunction)(void *)py_uwsgi_send_spool, METH_KEYWORDS, ""},
+	{"spool", (PyCFunction)(void *)py_uwsgi_send_spool, METH_KEYWORDS, ""},
 #endif
 	{"set_spooler_frequency", py_uwsgi_spooler_freq, METH_VARARGS, ""},
 	{"spooler_jobs", py_uwsgi_spooler_jobs, METH_VARARGS, ""},
@@ -2662,7 +2650,7 @@ static PyMethodDef uwsgi_advanced_methods[] = {
 
 	{"mule_msg", py_uwsgi_mule_msg, METH_VARARGS, ""},
 	{"farm_msg", py_uwsgi_farm_msg, METH_VARARGS, ""},
-	{"mule_get_msg", (PyCFunction) py_uwsgi_mule_get_msg, METH_VARARGS|METH_KEYWORDS, ""},
+	{"mule_get_msg", (PyCFunction)(void *)py_uwsgi_mule_get_msg, METH_VARARGS|METH_KEYWORDS, ""},
 	{"farm_get_msg", py_uwsgi_farm_get_msg, METH_VARARGS, ""},
 	{"in_farm", py_uwsgi_in_farm, METH_VARARGS, ""},
 
