@@ -175,8 +175,8 @@ static int http_parse(struct wsgi_request *wsgi_req, char *watermark) {
 	char *ptr = wsgi_req->proto_parser_buf;
 	char *base = ptr;
 	char *query_string = NULL;
-	char ip[INET_ADDRSTRLEN+1];
-	struct sockaddr_in *http_sin = (struct sockaddr_in *) &wsgi_req->c_addr;
+	char ip[INET6_ADDRSTRLEN+1];
+	struct sockaddr *http_sa = (struct sockaddr *) &wsgi_req->client_addr;
 	char *proxy_src = NULL;
 	char *proxy_dst = NULL;
 	char *proxy_src_port = NULL;
@@ -293,14 +293,56 @@ static int http_parse(struct wsgi_request *wsgi_req, char *watermark) {
 		}
 	}
 	else {
-		// TODO add ipv6 support
-		memset(ip, 0, INET_ADDRSTRLEN+1);
-		if (inet_ntop(AF_INET, (void *) &http_sin->sin_addr.s_addr, ip, INET_ADDRSTRLEN)) {
-			wsgi_req->uh->pktsize += proto_base_add_uwsgi_var(wsgi_req, "REMOTE_ADDR", 11, ip, strlen(ip));
+		// TODO log something useful for AF_UNIX sockets
+		switch(http_sa->sa_family) {
+		case AF_INET6: {
+			struct sockaddr_in6* http_sin = &wsgi_req->client_addr.sin6;
+			memset(ip, 0, sizeof(ip));
+			/* check if it's an IPv6-mapped-IPv4 address and, if so,
+			 * represent it as an IPv4 address
+			 *
+			 * these IPv6 macros are defined in POSIX.1-2001.
+			 */
+			if (IN6_IS_ADDR_V4MAPPED(&http_sin->sin6_addr)) {
+				/* just grab the last 4 bytes and pretend they're
+				 * IPv4. None of the word/half-word convenience
+				 * functions are in POSIX, so just stick to .s6_addr
+				 */
+				union {
+					unsigned char s6[4];
+					uint32_t s4;
+				} addr_parts;
+				memcpy(addr_parts.s6, &http_sin->sin6_addr.s6_addr[12], 4);
+				uint32_t in4_addr = addr_parts.s4;
+				if (inet_ntop(AF_INET, (void*)&in4_addr, ip, INET_ADDRSTRLEN)) {
+					wsgi_req->uh->pktsize += proto_base_add_uwsgi_var(wsgi_req, "REMOTE_ADDR", 11, ip, strlen(ip));
+				} else {
+					uwsgi_error("inet_ntop()");
+					return -1;
+				}
+			} else {
+				if (inet_ntop(AF_INET6, (void *) &http_sin->sin6_addr, ip, INET6_ADDRSTRLEN)) {
+					wsgi_req->uh->pktsize += proto_base_add_uwsgi_var(wsgi_req, "REMOTE_ADDR", 11, ip, strlen(ip));
+				} else {
+					uwsgi_error("inet_ntop()");
+					return -1;
+				}
+			}
+			break;
 		}
-		else {
-			uwsgi_error("inet_ntop()");
-			return -1;
+		case AF_INET:
+		default: {
+			struct sockaddr_in* http_sin = &wsgi_req->client_addr.sin;
+			memset(ip, 0, sizeof(ip));
+			if (inet_ntop(AF_INET, (void *) &http_sin->sin_addr, ip, INET_ADDRSTRLEN)) {
+				wsgi_req->uh->pktsize += proto_base_add_uwsgi_var(wsgi_req, "REMOTE_ADDR", 11, ip, strlen(ip));
+			}
+			else {
+				uwsgi_error("inet_ntop()");
+				return -1;
+			}
+			}
+			break;
 		}
 	}
 
@@ -708,21 +750,75 @@ int uwsgi_is_full_http(struct uwsgi_buffer *ub) {
 
 void uwsgi_proto_http_setup(struct uwsgi_socket *uwsgi_sock) {
 	uwsgi_sock->proto = uwsgi_proto_http_parser;
-                        uwsgi_sock->proto_accept = uwsgi_proto_base_accept;
-                        uwsgi_sock->proto_prepare_headers = uwsgi_proto_base_prepare_headers;
-                        uwsgi_sock->proto_add_header = uwsgi_proto_base_add_header;
-                        uwsgi_sock->proto_fix_headers = uwsgi_proto_base_fix_headers;
-                        uwsgi_sock->proto_read_body = uwsgi_proto_base_read_body;
-                        uwsgi_sock->proto_write = uwsgi_proto_base_write;
-                        uwsgi_sock->proto_writev = uwsgi_proto_base_writev;
-                        uwsgi_sock->proto_write_headers = uwsgi_proto_base_write;
-                        uwsgi_sock->proto_sendfile = uwsgi_proto_base_sendfile;
-                        uwsgi_sock->proto_close = uwsgi_proto_base_close;
-                        if (uwsgi.offload_threads > 0)
-                                uwsgi_sock->can_offload = 1;
+	uwsgi_sock->proto_accept = uwsgi_proto_base_accept;
+	uwsgi_sock->proto_prepare_headers = uwsgi_proto_base_prepare_headers;
+	uwsgi_sock->proto_add_header = uwsgi_proto_base_add_header;
+	uwsgi_sock->proto_fix_headers = uwsgi_proto_base_fix_headers;
+	uwsgi_sock->proto_read_body = uwsgi_proto_base_read_body;
+	uwsgi_sock->proto_write = uwsgi_proto_base_write;
+	uwsgi_sock->proto_writev = uwsgi_proto_base_writev;
+	uwsgi_sock->proto_write_headers = uwsgi_proto_base_write;
+	uwsgi_sock->proto_sendfile = uwsgi_proto_base_sendfile;
+	uwsgi_sock->proto_close = uwsgi_proto_base_close;
+	if (uwsgi.offload_threads > 0)
+		uwsgi_sock->can_offload = 1;
 
 }
 
+/*
+close the connection on errors, incomplete parsing, HTTP/1.0,  pipelined or offloaded requests
+NOTE: Connection: close is not honoured
+*/
+void uwsgi_proto_http11_close(struct wsgi_request *wsgi_req) {
+	// check for errors or incomplete packets
+	if (wsgi_req->write_errors || wsgi_req->proto_parser_status != 3 || wsgi_req->proto_parser_remains > 0
+		|| wsgi_req->post_pos < wsgi_req->post_cl || wsgi_req->via == UWSGI_VIA_OFFLOAD
+		|| !uwsgi_strncmp("HTTP/1.0", 8, wsgi_req->protocol, wsgi_req->protocol_len)) {
+		close(wsgi_req->fd);
+		wsgi_req->socket->retry[wsgi_req->async_id] = 0;
+		wsgi_req->socket->fd_threads[wsgi_req->async_id] = -1;
+	}
+	else {
+		wsgi_req->socket->retry[wsgi_req->async_id] = 1;
+		wsgi_req->socket->fd_threads[wsgi_req->async_id] = wsgi_req->fd;
+	}
+}
+
+int uwsgi_proto_http11_accept(struct wsgi_request *wsgi_req, int fd) {
+	if (wsgi_req->socket->retry[wsgi_req->async_id]) {
+		wsgi_req->fd = wsgi_req->socket->fd_threads[wsgi_req->async_id];
+		int ret = uwsgi_wait_read_req(wsgi_req);
+		if (ret <= 0) {
+			close(wsgi_req->fd);
+			wsgi_req->socket->retry[wsgi_req->async_id] = 0;
+			wsgi_req->socket->fd_threads[wsgi_req->async_id] = -1;
+			return -1;
+		}
+		return wsgi_req->socket->fd_threads[wsgi_req->async_id];
+	}
+	return uwsgi_proto_base_accept(wsgi_req, fd);
+}
+
+void uwsgi_proto_http11_setup(struct uwsgi_socket *uwsgi_sock) {
+	uwsgi_sock->proto = uwsgi_proto_http_parser;
+	uwsgi_sock->proto_accept = uwsgi_proto_http11_accept;
+	uwsgi_sock->proto_prepare_headers = uwsgi_proto_base_prepare_headers;
+	uwsgi_sock->proto_add_header = uwsgi_proto_base_add_header;
+	uwsgi_sock->proto_fix_headers = uwsgi_proto_base_fix_headers;
+	uwsgi_sock->proto_read_body = uwsgi_proto_base_read_body;
+	uwsgi_sock->proto_write = uwsgi_proto_base_write;
+	uwsgi_sock->proto_writev = uwsgi_proto_base_writev;
+	uwsgi_sock->proto_write_headers = uwsgi_proto_base_write;
+	uwsgi_sock->proto_sendfile = uwsgi_proto_base_sendfile;
+	uwsgi_sock->proto_close = uwsgi_proto_http11_close;
+	if (uwsgi.offload_threads > 0)
+		uwsgi_sock->can_offload = 1;
+	uwsgi_sock->fd_threads = uwsgi_malloc(sizeof(int) * uwsgi.cores);
+	memset(uwsgi_sock->fd_threads, -1, sizeof(int) * uwsgi.cores);
+	uwsgi_sock->retry = uwsgi_calloc(sizeof(int) * uwsgi.cores);
+	uwsgi.is_et = 1;
+}
+
 #ifdef UWSGI_SSL
 static int uwsgi_proto_https_parser(struct wsgi_request *wsgi_req) {
 
